@@ -140,6 +140,36 @@ class Ast2ToGAst(AstToGAst):
         new_node.end_lineno = new_node.end_col_offset = None
         return new_node
 
+    def visit_Subscript(self, node):
+        new_slice = self._visit(node.slice)
+        new_node = gast.Subscript(
+            self._visit(node.value),
+            new_slice,
+            self._visit(node.ctx),
+        )
+        gast.copy_location(new_node, node)
+        new_node.end_lineno = new_node.end_col_offset = None
+        return new_node
+
+    def visit_Ellipsis(self, node):
+        new_node = gast.Constant(
+            Ellipsis,
+            None,
+        )
+        gast.copy_location(new_node, node)
+        new_node.end_lineno = new_node.end_col_offset = None
+        return new_node
+
+    def visit_Index(self, node):
+        return self._visit(node.value)
+
+    def visit_ExtSlice(self, node):
+        new_dims = self._visit(node.dims)
+        new_node = gast.Tuple(new_dims, gast.Load())
+        gast.copy_location(new_node, node)
+        new_node.end_lineno = new_node.end_col_offset = None
+        return new_node
+
     def visit_Str(self, node):
         new_node = gast.Constant(
             node.s,
@@ -309,11 +339,35 @@ class GAstToAst2(GAstToAst):
     def visit_Constant(self, node):
         if isinstance(node.value, (bool, int, long, float, complex)):
             new_node = ast.Num(node.value)
+        elif node.value is Ellipsis:
+            new_node = ast.Ellipsis()
         else:
             new_node = ast.Str(node.value)
         ast.copy_location(new_node, node)
         return new_node
 
+    def visit_Subscript(self, node):
+        def adjust_slice(s):
+            if isinstance(s, (ast.Slice, ast.Ellipsis)):
+                return s
+            else:
+                return ast.Index(s)
+        if isinstance(node.slice, gast.Tuple):
+            new_slice = ast.ExtSlice([adjust_slice(self._visit(elt))
+                                      for elt in node.slice.elts])
+        else:
+            new_slice = adjust_slice(self._visit(node.slice))
+        ast.copy_location(new_slice, node.slice)
+
+        new_node = ast.Subscript(
+            self._visit(node.value),
+            new_slice,
+            self._visit(node.ctx),
+        )
+        ast.copy_location(new_node, node)
+        new_node.end_lineno = new_node.end_col_offset = None
+        return new_node
+
     def visit_Call(self, node):
         if node.args and isinstance(node.args[-1], gast.Starred):
             args = node.args[:-1]
